/* convert_life_pattern.pl */


/*
This program converts dot-and-star
Life patterns into Excelsior array
assignments. It is explained in
my Dobbs Code Talk posting 
"Prolog as a Text-Hacking Language".
*/


convert_life_pattern( InputFile, PatternName ) :-
  read_file_to_lines( InputFile, Lines ),
  lines_to_equations( Lines, PatternName, 4, Eqns ),
  atom_concat( PatternName, '.exc', OutputFile ),
  tell( OutputFile ),
  flatten( Eqns, EqnsFlattened ),
  maplist( write_equation, EqnsFlattened ),
  told.


lines_to_equations( [], _PatternName, _Y, [] ) :- !.

lines_to_equations( [ Line | Lines ], PatternName, Y, [ Eqns | MoreEqns ] ) :-
  line_to_equations( Line, PatternName, 4, Y, Eqns ),
  YPlus1 is Y+1,
  lines_to_equations( Lines, PatternName, YPlus1, MoreEqns ).


line_to_equations( [], _PatternName, _X, _Y, [] ) :- !.

line_to_equations( [ 0'* | Rest ], PatternName, X, Y, [ Eqn | Eqns ] ) :- !,
  Eqn = eqn( PatternName, X, Y ),
  XPlus1 is X+1,
  line_to_equations( Rest, PatternName, XPlus1, Y, Eqns ).

line_to_equations( [ 0'. | Rest ], PatternName, X, Y, Eqns ) :-
  XPlus1 is X+1,
  line_to_equations( Rest, PatternName, XPlus1, Y, Eqns ).


write_equation( eqn( PatternName, X, Y ) ) :-
  format( '~w[~w,~w] = "n".~n', [PatternName,X,Y] ). 


read_file_to_lines( File, Lines ) :-
  open( File, read, Stream, [] ),
  read_line_to_codes( Stream, Line ),
  read_stream_to_lines( Line, Stream, Lines ),
  close( Stream ).


read_stream_to_lines( end_of_file, _Stream, [] ) :- !.

read_stream_to_lines( Line, Stream, [ Line | Rest ] ) :- 
  read_line_to_codes( Stream, NextLine ),
  read_stream_to_lines( NextLine, Stream, Rest ).



